<?php

namespace Unific\Connector\Helper\Data;

use Magento\Framework\Exception\InputException;
use Symfony\Component\Config\Definition\Exception\Exception;
use Unific\Connector\Helper\Data;

class Cart extends \Magento\Framework\App\Helper\AbstractHelper
{
    protected $filterHelper;

    protected $customerSession;
    protected $checkoutSession;
    protected $customerHelper;

    protected $storeManager;
    protected $quoteIdMaskFactory;

    // Holds a Quote API DATA object
    private $cart;
    private $dataFormatter;

    protected $returnData = [
    ];

    /**
     * OrderPlugin constructor.
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Unific\Connector\Helper\Filter $filterHelper
     * @param \Magento\Customer\Model\Session $customerSession
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param Customer $customerHelper
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Quote\Model\QuoteIdMaskFactory $quoteIdMaskFactory
     * @param Formatter $dataFormatter
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Unific\Connector\Helper\Filter $filterHelper,
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Checkout\Model\Session $checkoutSession,
        Customer $customerHelper,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Quote\Model\QuoteIdMaskFactory $quoteIdMaskFactory,
        Formatter $dataFormatter
    ) {
        parent::__construct($context);

        $this->filterHelper = $filterHelper;

        $this->customerSession = $customerSession;
        $this->checkoutSession = $checkoutSession;
        $this->storeManager = $storeManager;
        $this->quoteIdMaskFactory = $quoteIdMaskFactory;
        $this->dataFormatter = $dataFormatter;
        $this->customerHelper = $customerHelper;
    }

    /**
     * @param \Magento\Quote\Model\Quote $cart
     * @param null $maskedQuoteId
     */
    public function setCart(\Magento\Quote\Model\Quote $cart, $maskedQuoteId = null)
    {
        $this->cart = $cart;
        $this->setCartInfo($maskedQuoteId);
    }

    public function setAddressData(\Magento\Quote\Model\Quote\Address $address)
    {
        $this->returnData['customer_email'] = $address->getEmail();
        $this->returnData['customer_firstname'] = $address->getFirstname();
        $this->returnData['customer_middlename'] = $address->getMiddlename();
        $this->returnData['customer_lastname'] = $address->getLastname();
        $this->returnData['customer_prefix'] = $address->getPrefix();
        $this->returnData['customer_suffix'] = $address->getSuffix();
        $this->returnData['customer_postcode'] = $address->getPostcode();

        $this->returnData = $this->dataFormatter->setStreetData($this->returnData, $address->getStreetFull(), 'customer_street');

        $this->returnData['customer_street'] = $address->getStreetFull();
        $this->returnData['customer_city'] = $address->getCity();
        $this->returnData['customer_telephone'] = $address->getTelephone();
        $this->returnData['customer_fax'] = $address->getFax();
        $this->returnData['customer_company'] = $address->getCompany();
        $this->returnData['customer_region'] = $address->getRegionCode();
        $this->returnData['customer_country'] = $address->getCountryModel()->getName();
    }

    /**
     * @return \Magento\Quote\Model\Data\QuoteInterface
     */
    public function getCart()
    {
        return $this->cart;
    }

    /**
     * @param null $maskedQuoteId
     * @return mixed
     */
    protected function setCartInfo($maskedQuoteId = null)
    {
        $this->returnData = $this->cart->getData();
        if($this->cart->getEntityId() != null && is_integer($this->cart->getEntityId()))
        {
            if($maskedQuoteId == null) {
                $quoteIdMask = $this->quoteIdMaskFactory->create()->load($this->cart->getEntityId(), 'quote_id');

                if ($quoteIdMask->getMaskedId() === null) {
                    $quoteIdMask->setQuoteId($this->cart->getEntityId())->save();
                    $maskedQuoteId = $quoteIdMask->getMaskedId();
                }
            }

            $this->returnData['masked_id'] = $maskedQuoteId;
            $baseUrl = $this->storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_WEB);
            $this->returnData['abandoned_checkout_url'] = $baseUrl . 'unific/cart/restore/id/' . $maskedQuoteId;

        }

        if ($this->customerSession->isLoggedIn()) {
            $this->returnData['customer_email'] = $this->customerSession->getCustomer()->getEmail();
            $this->returnData['customer_firstname'] = $this->customerSession->getCustomer()->getFirstname();
            $this->returnData['customer_middlename'] = $this->customerSession->getCustomer()->getMiddlename();
            $this->returnData['customer_lastname'] = $this->customerSession->getCustomer()->getLastname();
        }

        $this->returnData['items'] = [];
        foreach ($this->cart->getAllItems() as $item) {
            $this->returnData['items'][] = array_intersect_key($item->getData(), array_flip($this->filterHelper->getItemsWhitelist()));
        }

        // Trigger the setting of the customer
        if($this->customerSession->isLoggedIn() === true) {
            $this->customerHelper->setCustomer($this->customerSession->getCustomer()->getDataModel());
            $this->returnData['customer_is_guest'] = 0;
        } else {
            $this->customerHelper->generateGuestCustomer($this->cart);
            $this->returnData['customer_is_guest'] = 1;
        }

        /*
        $customerData = $this->customerHelper->getCustomerInfo();
        if($customerData['email'] != null)
        {
            $this->returnData['customer'] = $customerData;
        }
        */

        if(isset($this->returnData['customer'])) {
            unset($this->returnData['customer']);
        }
    }

    /**
     * @return array
     */
    public function getCartInfo()
    {
        // Sanitize Cart
        $this->returnData = array_intersect_key($this->returnData, array_flip($this->filterHelper->getCheckoutWhitelist()));
        return $this->filterHelper->sanitizeAddressData($this->returnData);
    }
}
