<?php

namespace Unific\Connector\Helper;

use Magento\Framework\Exception\InputException;
use Symfony\Component\Config\Definition\Exception\Exception;
use Magento\Store\Model\ScopeInterface;

class Historical extends \Magento\Framework\App\Helper\AbstractHelper
{
    protected $hmacKey = '';

    protected $queueHelper;
    protected $hmacHelper;

    protected $scopeConfig;

    protected $queueCollectionFactory;
    protected $historicalCollectionFactory;

    protected $historicalFactory;

    protected $searchCriteriaBuilder;

    protected $orderRepository;
    protected $customerRepository;

    protected $categoryFactory;
    protected $productFactory;

    protected $productMetadata;
    protected $moduleList;

    protected $orderDataHelper;
    protected $customerDataHelper;
    protected $cartDataHelper;
    protected $productDataHelper;
    protected $categoryDataHelper;

    protected $headerKey = 'unific/hmac/hmacHeader';

    /**
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Unific\Connector\Model\ResourceModel\Queue\CollectionFactory $queueCollectionFactory
     * @param \Unific\Connector\Model\ResourceModel\Historical\CollectionFactory $historicalCollectionFactory
     * @param \Unific\Connector\Model\HistoricalFactory $historicalFactory
     * @param Message\Queue $queueHelper
     * @param Hmac $hmacHelper
     * @param \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder
     * @param \Magento\Sales\Model\OrderRepository $orderRepository
     * @param \Magento\Customer\Model\ResourceModel\CustomerRepository $customerRepository
     * @param \Magento\Catalog\Model\ResourceModel\Category\CollectionFactory $categoryFactory
     * @param \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $productFactory
     * @param \Magento\Framework\App\ProductMetadataInterface $productMetadata
     * @param \Magento\Framework\Module\ModuleListInterface $moduleList
     * @param Data\Order $orderDataHelper
     * @param Data\Cart $cartDataHelper
     * @param Data\Customer $customerDataHelper
     * @param Data\Product $productDataHelper
     * @param Data\Category $categoryDataHelper
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Unific\Connector\Model\ResourceModel\Queue\CollectionFactory $queueCollectionFactory,
        \Unific\Connector\Model\ResourceModel\Historical\CollectionFactory $historicalCollectionFactory,
        \Unific\Connector\Model\HistoricalFactory $historicalFactory,
        Message\Queue $queueHelper,
        Hmac $hmacHelper,
        \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
        \Magento\Sales\Model\OrderRepository $orderRepository,
        \Magento\Customer\Model\ResourceModel\CustomerRepository $customerRepository,
        \Magento\Catalog\Model\ResourceModel\Category\CollectionFactory $categoryFactory,
        \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $productFactory,
        \Magento\Framework\App\ProductMetadataInterface $productMetadata,
        \Magento\Framework\Module\ModuleListInterface $moduleList,
        Data\Order $orderDataHelper,
        Data\Cart $cartDataHelper,
        Data\Customer $customerDataHelper,
        Data\Product $productDataHelper,
        Data\Category $categoryDataHelper
    ) {
        parent::__construct($context);

        $this->queueHelper = $queueHelper;
        $this->hmacHelper = $hmacHelper;

        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->orderRepository = $orderRepository;
        $this->customerRepository = $customerRepository;
        $this->productFactory = $productFactory;
        $this->categoryFactory = $categoryFactory;
        $this->queueCollectionFactory = $queueCollectionFactory;
        $this->historicalCollectionFactory = $historicalCollectionFactory;

        $this->productMetadata = $productMetadata;
        $this->moduleList = $moduleList;

        $this->historicalFactory = $historicalFactory;

        $this->orderDataHelper = $orderDataHelper;
        $this->customerDataHelper = $customerDataHelper;
        $this->cartDataHelper = $cartDataHelper;
        $this->productDataHelper = $productDataHelper;
        $this->categoryDataHelper = $categoryDataHelper;
    }

    /**
     * Trigger the historical process
     *
     * @return array
     */
    public function triggerHistorical()
    {
        $this->resetAllHistoricalData();

        $this->triggerHistoricalForType('customer');
        $this->triggerHistoricalForType('order');
        $this->triggerHistoricalForType('category');
        $this->triggerHistoricalForType('product');

        return ['message' => 'Historical data has been triggered'];
    }

    /**
     * Trigger historical for a specific type
     *
     * @param $type
     * @param bool $removeOld
     *
     * @return array
     */
    public function triggerHistoricalForType($type, $removeOld = false)
    {
        if($removeOld) {
            $this->removeHistoricalQueueType($type);
        }

        $historical = $this->historicalFactory->create();
        $historical->setHistoricalType($type);
        $historical->setHistoricalTypePage(1);
        $historical->save();

        return ['message' => 'Historical data for type [' . $type . '] has been triggered'];
    }

    /**
     * Delete a type from the historical queue process
     *
     * @param $type
     */
    public function removeHistoricalQueueType($type)
    {
        $collection = $this->historicalCollectionFactory->create();
        $collection->addFieldToFilter('historical_type', ['eq' => $type]);
        $collection->walk('delete');
    }

    /**
     * Inject historical data in the queue if historical is requested
     */
    public function queueHistorical()
    {
        $historicalCollection = $this->historicalCollectionFactory->create();

        if ($historicalCollection->getSize() > 0) {
            $searchCriteria = $this->searchCriteriaBuilder->create();
            $searchCriteria->setPageSize(Settings::HISTORICAL_PAGE_SIZE);

            foreach($historicalCollection as $historicalItem) {
                $items = [];

                $searchCriteria->setCurrentPage($historicalItem->getHistoricalQueuePage());
                $this->hmacKey = $this->scopeConfig->getValue($this->headerKey, ScopeInterface::SCOPE_STORE);

                if($historicalItem->getHistoricalType() == 'customer') {
                    // Queue Customers
                    $items = $this->customerRepository->getList($searchCriteria)->getItems();

                    foreach ($items as $customer) {
                        $this->customerDataHelper->setCustomer($customer);
                        $headers = $this->getHeaders('customer/historical', $this->customerDataHelper->getCustomerInfo());

                        $this->queueHelper->queue(
                            $this->scopeConfig->getValue('unific/webhook/customer_endpoint'),
                            $this->customerDataHelper->getCustomerInfo(),
                            Settings::PRIORITY_CUSTOMER,
                            $headers,
                            \Zend\Http\Request::METHOD_POST,
                            true
                        );
                    }
                }

                if($historicalItem->getHistoricalType() == 'order') {
                    // Queue Orders
                    $items = $this->orderRepository->getList($searchCriteria);

                    foreach ($items as $order) {
                        $this->orderDataHelper->setOrder($order);
                        $headers = $this->getHeaders('order/historical', $this->orderDataHelper->getOrderInfo());

                        $this->queueHelper->queue(
                            $this->scopeConfig->getValue('unific/webhook/order_endpoint'),
                            $this->orderDataHelper->getOrderInfo(),
                            Settings::PRIORITY_ORDER,
                            $headers,
                            \Zend\Http\Request::METHOD_POST,
                            true
                        );
                    }
                }

                if($historicalItem->getHistoricalType() == 'category') {
                    // Queue Categories
                    $items = $this->categoryFactory->create()
                        ->addAttributeToSelect('*')
                        ->setPageSize(Settings::HISTORICAL_PAGE_SIZE)
                        ->setCurPage($historicalItem->getHistoricalTypePage());

                    foreach ($items as $category) {
                        $this->categoryDataHelper->setCategory($category);
                        $headers = $this->getHeaders('category/historical', $this->categoryDataHelper->getCategoryInfo());

                        $extraHeaders = [];
                        $extraHeaders['X-SUBJECT'] = 'category/historical';
                        $extraHeaders[$this->hmacKey] = $this->hmacHelper->generateHmac($this->categoryDataHelper->getCategoryInfo());

                        $this->queueHelper->queue(
                            $this->scopeConfig->getValue('unific/webhook/category_endpoint'),
                            $this->categoryDataHelper->getCategoryInfo(),
                            Settings::PRIORITY_CATEGORY,
                            $headers,
                            \Zend\Http\Request::METHOD_POST,
                            true
                        );
                    }
                }

                if($historicalItem->getHistoricalType() == 'product') {
                    // Queue Products
                    $items = $this->productFactory->create()
                        ->addAttributeToSelect('*')
                        ->setPageSize(Settings::HISTORICAL_PAGE_SIZE)
                        ->setCurPage($historicalItem->getHistoricalTypePage());

                    foreach ($items as $product) {
                        $this->productDataHelper->setProduct($product);
                        $headers = $this->getHeaders('product/historical', $this->productDataHelper->getProductInfo());

                        $this->queueHelper->queue(
                            $this->scopeConfig->getValue('unific/webhook/product_endpoint'),
                            $this->productDataHelper->getProductInfo(),
                            Settings::PRIORITY_PRODUCT,
                            $headers,
                            \Zend\Http\Request::METHOD_POST,
                            true
                        );
                    }
                }

                // Write back the latest ID to the historical queue manager
                if(count($items) == Settings::HISTORICAL_PAGE_SIZE) {
                    $this->incrementHistoricalQueuePage($historicalItem->getHistoricalType());
                } else {
                    $this->removeHistoricalQueueType($historicalItem->getHistoricalType());
                }
            }
        }
    }

    /**
     * Queue up all the historical data to be ready for sending
     */
    public function resetAllHistoricalData()
    {
        $collection = $this->historicalCollectionFactory->create();
        $collection->walk('delete');

        $queueCollection = $this->queueCollectionFactory->create();
        $queueCollection->addFieldToFilter('headers', ['like' => '%/historical%']);
        $queueCollection->walk('delete');

        return ['message' => 'Historical data has been stopped and reset'];
    }

    /**
     * Queue up all the historical data to be ready for sending
     * @param $type
     * @return array
     */
    public function resetHistoricalForType($type)
    {
        $this->removeHistoricalQueueType($type);

        $queueCollection = $this->queueCollectionFactory->create();
        $queueCollection->addFieldToFilter('headers', ['like' => '%' . $type . '/historical%']);

        $queueCollection->walk('delete');

        return ['message' => 'Historical data for type [' . $type . '] has been stopped and reset'];
    }

    /**
     * Update the latest queued ID
     *
     * @param $type
     */
    protected function incrementHistoricalQueuePage($type)
    {
        $collection = $this->historicalCollectionFactory->create();
        $collection->addFieldToFilter('historical_type', ['eq', $type]);

        foreach($collection as $item) {
            $item->setHistoricalTypePage($item->getHistoricalTypePage() + 1);
        }

        $collection->save();
    }

    /**
     * @param $subject
     * @param $data
     * @return array
     */
    protected function getHeaders($subject)
    {
        $headers = [];
        $headers['X-SUBJECT'] = $subject;

        return $headers;
    }
}
