<?php

namespace Unific\Connector\Plugin;

use Unific\Connector\Helper\Settings;

class CartPlugin extends AbstractPlugin
{
    protected $cartDataHelper;
    protected $cartRepository;
    protected $customerSession;

    /**
     * @param \Unific\Connector\Helper\Data\Cart $cartDataHelper
     * @param \Magento\Quote\Api\CartRepositoryInterface $cartRepository
     * @param \Magento\Customer\Model\Session $customerSession
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Unific\Connector\Helper\Hmac $hmacHelper
     * @param \Unific\Connector\Helper\Message\Queue $queueHelper
     * @param \Magento\Framework\App\ProductMetadataInterface $productMetadata
     * @param \Magento\Framework\Module\ModuleListInterface $moduleList
     */
    public function __construct(
        \Unific\Connector\Helper\Data\Cart $cartDataHelper,
        \Magento\Quote\Api\CartRepositoryInterface $cartRepository,
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Unific\Connector\Helper\Hmac $hmacHelper,
        \Unific\Connector\Helper\Message\Queue $queueHelper,
        \Magento\Framework\App\ProductMetadataInterface $productMetadata,
        \Magento\Framework\Module\ModuleListInterface $moduleList
    ) {
        parent::__construct(
            $scopeConfig,
            $hmacHelper,
            $queueHelper,
            $productMetadata,
            $moduleList
        );

        $this->cartDataHelper = $cartDataHelper;
        $this->cartRepository = $cartRepository;
        $this->customerSession = $customerSession;
    }

    /**
     * @param $subject
     * @return array
     */
    public function afterSave($subject)
    {
        if ($this->scopeConfig->getValue('unific/connector/enabled') == 1) {
            $this->cartDataHelper->setCart($subject->getQuote());

            $integrationSubject = 'checkout/create';

            // Magento adds the created at tag, if the created date differs from the current change
            if ($this->cartDataHelper->getCart()->getCreatedAt() != null) {
                $integrationSubject = 'checkout/update';
            }

            $this->processWebhook(
                $this->cartDataHelper->getCartInfo(),
                $this->scopeConfig->getValue('unific/webhook/cart_endpoint'),
                Settings::PRIORITY_CART,
                $integrationSubject
            );
        }

        return $subject;
    }
}
