<?php

namespace Unific\Connector\Plugin;

use Unific\Connector\Helper\Settings;

class CustomerAddressPlugin extends AbstractPlugin
{
    protected $customerDataHelper;
    protected $customerRepository;

    /**
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Unific\Connector\Helper\Hmac $hmacHelper
     * @param \Unific\Connector\Helper\Message\Queue $queueHelper
     * @param \Magento\Framework\App\ProductMetadataInterface $productMetadata
     * @param \Magento\Framework\Module\ModuleListInterface $moduleList
     * @param \Unific\Connector\Helper\Data\Customer $customerDataHelper
     * @param \Magento\Customer\Api\CustomerRepositoryInterface $customerRepository
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Unific\Connector\Helper\Hmac $hmacHelper,
        \Unific\Connector\Helper\Message\Queue $queueHelper,
        \Magento\Framework\App\ProductMetadataInterface $productMetadata,
        \Magento\Framework\Module\ModuleListInterface $moduleList,
        \Unific\Connector\Helper\Data\Customer $customerDataHelper,
        \Magento\Customer\Api\CustomerRepositoryInterface $customerRepository
    ) {
        parent::__construct(
            $scopeConfig,
            $hmacHelper,
            $queueHelper,
            $productMetadata,
            $moduleList
        );

        $this->customerDataHelper = $customerDataHelper;
        $this->customerRepository = $customerRepository;
    }

    /**
     * @param $subject
     * @param $address
     * @return mixed
     */
    public function afterSave($subject, $address)
    {
        if ($this->scopeConfig->getValue('unific/connector/enabled') == 1 && $address->getCustomerId() > 0) {

            $customer = $this->customerRepository->getById($address->getCustomerId());

            if ($customer->getId() !== null) {
                $customer->setDefaultBilling($address->getId());
                $customer->setDefaultShipping($address->getId());
                $customer->setAddresses(array($address));

                $this->customerDataHelper->setCustomer($customer);

                if(filter_var($customer->getEmail(), FILTER_VALIDATE_EMAIL)) {
                    $this->processWebhook(
                        $this->customerDataHelper->getCustomerInfo(),
                        $this->scopeConfig->getValue('unific/webhook/customer_endpoint'),
                        Settings::PRIORITY_CUSTOMER,
                        'customer/update'
                    );
                }
            }
        }

        return $address;
    }
}
