<?php

namespace Unific\Connector\Plugin;

use Unific\Connector\Helper\Settings;

class CustomerUpdatePlugin extends AbstractPlugin
{
    protected $customerDataHelper;
    protected $customerRepository;

    /**
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Unific\Connector\Helper\Hmac $hmacHelper
     * @param \Unific\Connector\Helper\Message\Queue $queueHelper
     * @param \Magento\Framework\App\ProductMetadataInterface $productMetadata
     * @param \Magento\Framework\Module\ModuleListInterface $moduleList
     * @param \Unific\Connector\Helper\Data\Customer $customerDataHelper
     * @param \Magento\Customer\Api\CustomerRepositoryInterface $customerRepository
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Unific\Connector\Helper\Hmac $hmacHelper,
        \Unific\Connector\Helper\Message\Queue $queueHelper,
        \Magento\Framework\App\ProductMetadataInterface $productMetadata,
        \Magento\Framework\Module\ModuleListInterface $moduleList,
        \Unific\Connector\Helper\Data\Customer $customerDataHelper,
        \Magento\Customer\Api\CustomerRepositoryInterface $customerRepository
    ) {
        parent::__construct(
            $scopeConfig,
            $hmacHelper,
            $queueHelper,
            $productMetadata,
            $moduleList
        );

        $this->customerDataHelper = $customerDataHelper;
        $this->customerRepository = $customerRepository;
    }

    /**
     * @param $subject
     * @param $customer
     * @return mixed
     */
    public function afterSave($subject, $customer)
    {
        if ($this->scopeConfig->getValue('unific/connector/enabled') == 1 && $customer->getId() > 0) {
            // Only send if the date created is more than 5 seconds different from the updated date
            // Sometimes a second save action happens within a second after creation
            if ($customer->getUpdatedAt() != "" && strtotime($customer->getUpdatedAt()) - strtotime($customer->getCreatedAt()) > 5) {
                $this->customerDataHelper->setCustomer($customer);

                if (filter_var($customer->getEmail(), FILTER_VALIDATE_EMAIL)) {
                    $this->processWebhook(
                        $this->customerDataHelper->getCustomerInfo(),
                        $this->scopeConfig->getValue('unific/webhook/customer_endpoint'),
                        Settings::PRIORITY_CUSTOMER,
                        'customer/update'
                    );
                }
            }
        }

        return $customer;
    }
}
